@@ -4,8 +4,9 @@ from typing import Collection, Dict, List, Mapping, Optional, Tuple
 
 from pydantic import parse_raw_as
 
-from metaphor.common.filter import DatabaseFilter, DatasetFilter
+from metaphor.common.filter import DatasetFilter
 from metaphor.common.query_history import chunk_query_logs
+from metaphor.common.tag_matcher import tag_datasets
 from metaphor.common.utils import chunks, md5_digest, start_of_day
 from metaphor.snowflake.accessed_object import AccessedObject
 
@@ -54,7 +55,7 @@ from metaphor.snowflake.utils import (
 logger = get_logger()
 
 # Filter out "Snowflake" database & all "information_schema" schemas
-DEFAULT_FILTER: DatabaseFilter = DatasetFilter(
+DEFAULT_FILTER: DatasetFilter = DatasetFilter(
     excludes={
         "SNOWFLAKE": None,
         "*": {"INFORMATION_SCHEMA": None},
@@ -76,19 +77,21 @@ class SnowflakeExtractor(BaseExtractor):
         super().__init__(config, "Snowflake metadata crawler", Platform.SNOWFLAKE)
         self._account = config.account
         self._filter = config.filter.normalize().merge(DEFAULT_FILTER)
+        self._tag_matchers = config.tag_matchers
         self._query_log_excluded_usernames = config.query_log.excluded_usernames
         self._query_log_lookback_days = config.query_log.lookback_days
         self._query_log_fetch_size = config.query_log.fetch_size
         self._max_concurrency = config.max_concurrency
+        self._config = config
 
-        self._conn = auth.connect(config)
         self._datasets: Dict[str, Dataset] = {}
         self._logs: List[QueryLog] = []
 
     async def extract(self) -> Collection[ENTITY_TYPES]:
-
         logger.info("Fetching metadata from Snowflake")
 
+        self._conn = auth.connect(self._config)
+
         with self._conn:
             cursor = self._conn.cursor()
 
@@ -126,8 +129,11 @@ class SnowflakeExtractor(BaseExtractor):
             if self._query_log_lookback_days > 0:
                 self._fetch_query_logs()
 
+        datasets = list(self._datasets.values())
+        tag_datasets(datasets, self._tag_matchers)
+
         entities: List[ENTITY_TYPES] = []
-        entities.extend(self._datasets.values())
+        entities.extend(datasets)
         entities.extend(chunk_query_logs(self._logs))
         return entities
 
@@ -237,7 +243,7 @@ class SnowflakeExtractor(BaseExtractor):
     def _fetch_table_info(
         self, tables: Dict[str, DatasetInfo], is_shared_database: bool
     ) -> None:
-        dict_cursor = self._conn.cursor(DictCursor)
+        dict_cursor: DictCursor = self._conn.cursor(DictCursor)  # type: ignore
         for chunk in chunks(list(tables.items()), TABLE_INFO_FETCH_SIZE):
             self._fetch_table_info_internal(dict_cursor, chunk, is_shared_database)
 
