@@ -1,4 +1,4 @@
-from typing import Collection, Dict, Iterable, List
+from typing import Collection, Dict, Iterable, List, Optional
 
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.logger import get_logger
@@ -32,6 +32,7 @@ class SynapseExtractor(MssqlExtractor):
             config, "Synapse metadata crawler", Platform.SYNAPSE, DataPlatform.SYNAPSE
         )
         self._config = config
+        self._filter = config.filter.normalize()
         self._lookback_days = config.query_log.lookback_days if config.query_log else 0
 
     async def extract(self) -> Collection[ENTITY_TYPES]:
@@ -57,6 +58,8 @@ class SynapseExtractor(MssqlExtractor):
         # Serverless sqlpool
         try:
             for database in serverless_client.get_databases():
+                if not self._filter.include_database(database.name):
+                    continue
                 tables = serverless_client.get_tables(database.name)
                 datasets = self._map_tables_to_dataset(
                     self._config.server_name, database, tables
@@ -80,6 +83,8 @@ class SynapseExtractor(MssqlExtractor):
         # Dedicated sqlpool
         try:
             for database in dedicated_client.get_databases():
+                if not self._filter.include_database(database.name):
+                    continue
                 tables = dedicated_client.get_tables(database.name)
                 datasets = self._map_tables_to_dataset(
                     self._config.server_name, database, tables
@@ -105,7 +110,9 @@ class SynapseExtractor(MssqlExtractor):
         entities.extend(chunk_query_logs(querylog_list))
         return entities
 
-    def _map_query_log(self, rows: Iterable[SynapseQueryLog], database: str = None):
+    def _map_query_log(
+        self, rows: Iterable[SynapseQueryLog], database: Optional[str] = None
+    ):
         querylog_map: Dict[str, QueryLog] = {}
         for row in rows:
             query_id = (
@@ -121,7 +128,11 @@ class SynapseExtractor(MssqlExtractor):
             query_id = generate_querylog_id(DataPlatform.SYNAPSE.name, row.request_id)
             queryLog.id = query_id
             queryLog.query_id = row.request_id
-            queryLog.type = self._map_query_type(row.query_operation)
+            queryLog.type = (
+                self._map_query_type(row.query_operation)
+                if row.query_operation
+                else None
+            )
             queryLog.platform = DataPlatform.SYNAPSE
             queryLog.start_time = row.start_time
             queryLog.duration = row.duration / 1000.0
@@ -138,7 +149,7 @@ class SynapseExtractor(MssqlExtractor):
             querylog_map[query_id] = queryLog
         return querylog_map.values()
 
-    def _map_query_type(self, operation: str) -> TypeEnum:
+    def _map_query_type(self, operation: str) -> Optional[TypeEnum]:
         operation = operation.upper()
         if operation in ["CREATE", "DROP", "ALTER", "TRUNCATE"]:
             return TypeEnum.DDL
