@@ -2,7 +2,7 @@ import base64
 import json
 import re
 import traceback
-from typing import Collection, Dict, List, Optional, Set
+from typing import Collection, Dict, List, Optional, Set, Union
 
 try:
     import tableauserverclient as tableau
@@ -36,6 +36,7 @@ from metaphor.models.metadata_change_event import (
     VirtualViewType,
 )
 from metaphor.tableau.config import TableauRunConfig
+from metaphor.tableau.graphql_utils import paginate_connection
 from metaphor.tableau.query import (
     CustomSqlTable,
     DatabaseTable,
@@ -81,6 +82,7 @@ class TableauExtractor(BaseExtractor):
     async def extract(self) -> Collection[ENTITY_TYPES]:
         logger.info("Fetching metadata from Tableau")
 
+        tableau_auth: Union[tableau.PersonalAccessTokenAuth, tableau.TableauAuth]
         if self._access_token is not None:
             tableau_auth = tableau.PersonalAccessTokenAuth(
                 self._access_token.token_name,
@@ -93,6 +95,10 @@ class TableauExtractor(BaseExtractor):
                 self._user_password.password,
                 self._site_name,
             )
+        else:
+            raise Exception(
+                "Must provide either access token or user password in config"
+            )
 
         server = tableau.Server(self._server_url, use_server_version=True)
         with server.auth.sign_in(tableau_auth):
@@ -108,11 +114,14 @@ class TableauExtractor(BaseExtractor):
         logger.info(
             f"There are {len(views)} views on site: {[view.name for view in views]}\n"
         )
-        for item in views:
-            logger.debug(json.dumps(item.__dict__, default=str))
+        for view in views:
+            logger.debug(json.dumps(view.__dict__, default=str))
             if not self._disable_preview_image:
-                server.views.populate_preview_image(item)
-            self._views[item.id] = item
+                server.views.populate_preview_image(view)
+            if not view.id:
+                logger.exception(f"view {view.name} missing id")
+                continue
+            self._views[view.id] = view
 
         # fetch all workbooks
         workbooks: List[tableau.WorkbookItem] = list(tableau.Pager(server.workbooks))
@@ -120,23 +129,23 @@ class TableauExtractor(BaseExtractor):
         logger.info(
             f"\nThere are {len(workbooks)} workbooks on site: {[workbook.name for workbook in workbooks]}"
         )
-        for item in workbooks:
-            server.workbooks.populate_views(item, usage=True)
-            logger.debug(json.dumps(item.__dict__, default=str))
+        for workbook in workbooks:
+            server.workbooks.populate_views(workbook, usage=True)
+            logger.debug(json.dumps(workbook.__dict__, default=str))
 
             try:
-                self._parse_dashboard(item)
+                self._parse_dashboard(workbook)
             except Exception as error:
                 traceback.print_exc()
-                logger.error(f"failed to parse workbook {item.name}, error {error}")
+                logger.error(f"failed to parse workbook {workbook.name}, error {error}")
 
     def _extract_datasources(self, server: tableau.Server) -> None:
-        # fetch custom SQL tablesfrom Metadata GraphQL API
-        resp = server.metadata.query(custom_sql_graphql_query)
-        resp_data = resp["data"]
-        json_dump_to_debug_file(resp_data, "graphql_custom_sql_tables.json")
+        # fetch custom SQL tables from Metadata GraphQL API
+        custom_sql_tables = paginate_connection(
+            server, custom_sql_graphql_query, "customSQLTablesConnection"
+        )
 
-        custom_sql_tables = resp_data["customSQLTables"]
+        json_dump_to_debug_file(custom_sql_tables, "graphql_custom_sql_tables.json")
         logger.info(f"Found {len(custom_sql_tables)} custom SQL tables.")
 
         datasource_upstream_datasets = {}
@@ -147,11 +156,13 @@ class TableauExtractor(BaseExtractor):
             )
 
         # fetch workbook related info from Metadata GraphQL API
-        resp = server.metadata.query(workbooks_graphql_query)
-        resp_data = resp["data"]
-        json_dump_to_debug_file(resp_data, "graphql_workbooks.json")
+        workbooks = paginate_connection(
+            server, workbooks_graphql_query, "workbooksConnection"
+        )
+        json_dump_to_debug_file(workbooks, "graphql_workbooks.json")
+        logger.info(f"Found {len(workbooks)} workbooks.")
 
-        for item in resp_data["workbooks"]:
+        for item in workbooks:
             try:
                 workbook = WorkbookQueryResponse.parse_obj(item)
                 self._parse_workbook_query_response(
@@ -163,10 +174,14 @@ class TableauExtractor(BaseExtractor):
                 )
 
     def _parse_dashboard(self, workbook: tableau.WorkbookItem) -> None:
+        if not workbook.webpage_url:
+            logger.exception(f"workbook {workbook.name} missing webpage_url")
+            return
+
         workbook_id = TableauExtractor._extract_workbook_id(workbook.webpage_url)
 
         views: List[tableau.ViewItem] = workbook.views
-        charts = [self._parse_chart(self._views[view.id]) for view in views]
+        charts = [self._parse_chart(self._views[view.id]) for view in views if view.id]
         total_views = sum([view.total_views for view in views])
 
         dashboard_info = DashboardInfo(
@@ -258,67 +273,71 @@ class TableauExtractor(BaseExtractor):
         source_virtual_views: List[str] = []
         published_datasources: List[str] = []
 
-        for source in workbook.upstreamDatasources:
+        for published_source in workbook.upstreamDatasources:
             virtual_view_id = str(
                 to_virtual_view_entity_id(
-                    source.luid, VirtualViewType.TABLEAU_DATASOURCE
+                    published_source.luid, VirtualViewType.TABLEAU_DATASOURCE
                 )
             )
-            if source.luid in self._virtual_views:
+            if published_source.luid in self._virtual_views:
                 # data source already parsed
                 source_virtual_views.append(virtual_view_id)
-                published_datasources.append(source.name)
+                published_datasources.append(published_source.name)
                 continue
 
             # Use the upstream datasets parsed from custom SQL if available
             source_datasets = datasource_upstream_datasets.get(
-                source.id, self._parse_upstream_datasets(source.upstreamTables)
+                published_source.id,
+                self._parse_upstream_datasets(published_source.upstreamTables),
             )
 
-            self._virtual_views[source.luid] = VirtualView(
+            self._virtual_views[published_source.luid] = VirtualView(
                 logical_id=VirtualViewLogicalID(
-                    type=VirtualViewType.TABLEAU_DATASOURCE, name=source.luid
+                    type=VirtualViewType.TABLEAU_DATASOURCE, name=published_source.luid
                 ),
                 tableau_datasource=TableauDatasource(
-                    name=f"{workbook.projectName}.{source.name}",
-                    description=source.description or None,
+                    name=f"{workbook.projectName}.{published_source.name}",
+                    description=published_source.description or None,
                     fields=[
                         TableauField(field=f.name, description=f.description or None)
-                        for f in source.fields
+                        for f in published_source.fields
                     ],
                     embedded=False,
-                    url=f"{self._base_url}/datasources/{source.vizportalUrlId}",
+                    url=f"{self._base_url}/datasources/{published_source.vizportalUrlId}",
                     source_datasets=source_datasets or None,
                 ),
             )
             source_virtual_views.append(virtual_view_id)
-            published_datasources.append(source.name)
+            published_datasources.append(published_source.name)
 
-        for source in workbook.embeddedDatasources:
-            if source.name in published_datasources:
+        for embedded_source in workbook.embeddedDatasources:
+            if embedded_source.name in published_datasources:
                 logger.debug(
-                    f"Skip embedded datasource {source.name} since it's published"
+                    f"Skip embedded datasource {embedded_source.name} since it's published"
                 )
                 continue
 
             virtual_view_id = str(
-                to_virtual_view_entity_id(source.id, VirtualViewType.TABLEAU_DATASOURCE)
+                to_virtual_view_entity_id(
+                    embedded_source.id, VirtualViewType.TABLEAU_DATASOURCE
+                )
             )
 
             # Use the upstream datasets parsed from custom SQL if available
             source_datasets = datasource_upstream_datasets.get(
-                source.id, self._parse_upstream_datasets(source.upstreamTables)
+                embedded_source.id,
+                self._parse_upstream_datasets(embedded_source.upstreamTables),
             )
 
-            self._virtual_views[source.id] = VirtualView(
+            self._virtual_views[embedded_source.id] = VirtualView(
                 logical_id=VirtualViewLogicalID(
-                    type=VirtualViewType.TABLEAU_DATASOURCE, name=source.id
+                    type=VirtualViewType.TABLEAU_DATASOURCE, name=embedded_source.id
                 ),
                 tableau_datasource=TableauDatasource(
-                    name=f"{workbook.projectName}.{source.name}",
+                    name=f"{workbook.projectName}.{embedded_source.name}",
                     fields=[
                         TableauField(field=f.name, description=f.description or None)
-                        for f in source.fields
+                        for f in embedded_source.fields
                     ],
                     embedded=True,
                     source_datasets=source_datasets or None,
@@ -345,7 +364,12 @@ class TableauExtractor(BaseExtractor):
         )
 
     def _parse_dataset_id(self, table: DatabaseTable) -> Optional[EntityId]:
-        if None in (table.name, table.schema, table.fullName, table.database):
+        if (
+            not table.name
+            or not table.schema_
+            or not table.fullName
+            or not table.database
+        ):
             return None
 
         database_name = table.database.name
@@ -420,11 +444,14 @@ class TableauExtractor(BaseExtractor):
 
         return match.group(1)
 
-    def _build_view_url(self, content_url: str) -> Optional[str]:
+    def _build_view_url(self, content_url: Optional[str]) -> Optional[str]:
         """
         Builds view URL from the API content_url field.
         content_url is in the form of <workbook>/sheets/<view>, e.g. 'Superstore/sheets/WhatIfForecast'
         """
+        if not content_url:
+            return None
+
         workbook, _, view = content_url.split("/")
 
         return f"{self._base_url}/views/{workbook}/{view}"
